; ==============================================================================
; アナログボタン読み取り処理
; ==============================================================================

.EQU BTN_NUM = 10
.EQU BTN_EXTRA = 0


; ボタン電圧間隔(16bit AD変換値 * 256)
#define BTN_VSTEP INT(0.5 + 255.0 * 256 / BTN_NUM)
; ボタン電圧閾値初期値(VSTEP の 1/2 と四捨五入オフセット 0x80)
#define BTN_VTH_INITIAL INT(BTN_VSTEP / 2 + 0x80)


; ボタン制御用レジスタ
.DEF LASTBTN1 = R16
.DEF LASTBTN2 = R17
.DEF ADVALUE = R18
.DEF VTH_L = R19
.DEF VTH_H = R20

	; ==============================================================================
	; ボタン読み取り処理
	; ARG 押されたボタン番号 0～20 または 0xff
	; 破壊: TMP, LASTBTN1, LASTBTN1, ADVALUE, X
	; ==============================================================================
SUB_READ_BUTTON:
	; ADC 電源ON
	ldi TMP, PRR_ADC_ON
	out PRR, TMP

	; ボタン読み取り(同じ値が 3連続で読めるまで繰り返し)
	ldi ARG, 0xfe
SCAN_LP1:
	mov LASTBTN2, LASTBTN1
	mov LASTBTN1, ARG
	rcall SUB_ADC_BUTTON

	; 同じ値が3度読めたかチェック
	cpse ARG, LASTBTN2
	rjmp SCAN_LP1
	cpse ARG, LASTBTN1
	rjmp SCAN_LP1

	ret


	; ==============================================================================
	; ボタン読み取り
	; ARG 押されたボタン番号 0～20 または 0xff
	; 破壊: TMP, ADVALUE, X
	; ==============================================================================
SUB_ADC_BUTTON:
	; BTNIN をアナログ入力へ
	sbi DIDR0, PNO_BTNIN	; BTNIN デジタル入力禁止
	sbi PORTB, PNO_BTNOUT	; BTNOUT を H へ(読み取り電圧出力)
	cbi PUEB, PNO_BTNIN		; BTNIN プルアップ停止

	; SLEEPモード = AD変換
	ldi TMP, SMCR_ADC
	out CCP, CONST_CCPID
	out SMCR, TMP

	; AD変換
	ldi TMP, ADCSRA_ADC_ON
	out ADCSRA, TMP
	sleep	; 変換開始

	; BTNIN をデジタル入力へ
	sbi PUEB, PNO_BTNIN		; BTNIN プルアップ
	cbi PORTB, PNO_BTNOUT	; BTNOUT を L へ(キーリリース検出)
	cbi DIDR0, PNO_BTNIN	; BTNIN デジタル入力

	in ADVALUE, ADCL ; 変換値取得

	; ADC OFF
	cbi ADCSRA, ADEN

	; ボタンが離されてないことの確認
	ldi ARG, 0xff
	sbic PINB, PNO_BTNIN
	ret	; ボタンリリース

	// 閾値スキャン
	ldi VTH_L, LOW(BTN_VTH_INITIAL)
	ldi VTH_H, HIGH(BTN_VTH_INITIAL)
BTN_SCAN:
	inc ARG
	cp VTH_H, ADVALUE
	brcc BTN_RET
	; 即値足し算を即値引き算で代用
	;addi VTH_L, LOW(BTN_VSTEP)
	;adci VTH_H, HIGH(BTN_VSTEP)
	;brcc BTN_SCAN
	subi VTH_L, -LOW(BTN_VSTEP)
	sbci VTH_H, -HIGH(BTN_VSTEP) - 1
	brcs BTN_SCAN
	inc ARG		; エクストラボタン番号(使用の有無によらず)
BTN_RET:
	ret


.UNDEF ADVALUE
.UNDEF LASTBTN1
.UNDEF LASTBTN2
.UNDEF VTH_L
.UNDEF VTH_H


