;
; TinyBattleSound
;

#define ADDR(l) (MAPPED_FLASH_START + 2 * l)

; ピン定義
; PB0	LED(HZ/H /MISO)			サウンド出力
; PB1	BTNOUT(OC0B/HZ /SCK)	ボタン読み取り電圧出力ピン（常にプルダウンされるラインなので、プルダウンに強い SCKピンに割り当てる）
; PB2	BTNIN(INT0/ADC2)		ボタン入力には INT0 が使える必要があるのでここ
; PB3	RESET
.EQU PNO_SOUND = 0
.EQU PNO_BTNOUT = 1
.EQU PNO_BTNIN = 2
.EQU PNO_RESET = 3

; 電力制御
.EQU PRR_OFF		= (1 << PRADC) | (1 << PRTIM0)	; ADC:OFF TIM0:OFF
.EQU PRR_ADC_ON		= (0 << PRADC) | (1 << PRTIM0)	; ADC:ON  TOM0:OFF
.EQU PRR_TIM_ON		= (1 << PRADC) | (0 << PRTIM0)	; ADC:OFF TOM0:ON

; システムクロック
.EQU CLKPSR_CLK	= 0x04			; 0100 1/16 = 500kHz
.EQU CLKPSR_CLK_LOW	= 0x06		; 0110 1/64 = 125kHz

; ADC設定 CLKPSR_CLK_SCANBTN の 1/4 125kHz
.EQU ADCSRA_ADC_OFF	= (0 << ADEN) | (0 << ADSC) | (0 << ADATE) | (1 << ADIF) | (0 << ADIE) | (0 << ADPS2) | (1 << ADPS1) | (0 << ADPS0)
.EQU ADCSRA_ADC_ON	= (1 << ADEN) | (0 << ADSC) | (0 << ADATE) | (1 << ADIF) | (1 << ADIE) | (0 << ADPS2) | (1 << ADPS1) | (0 << ADPS0)

; SLEEPモード
.EQU SMCR_ADC		= (0 << SM2) | (0 << SM1) | (1 << SM0) | (1 << SE)
.EQU SMCR_POWERDOWN	= (0 << SM2) | (1 << SM1) | (0 << SM0) | (1 << SE)


; レジスタ
.DEF ARG = R23
.DEF CONST_CCPID = R24
.DEF ZERO = R25
#define TMP YL
#define TMP2 YH

.CSEG
.ORG 0
	; 未使用割り込みベクタをウェイト処理に使い回し

	rjmp	RESET					; 1	RESET
	reti							; 2 INT0 

WAIT_4X_14:
WAIT_4X_R12:
	nop								; 3 PCINT0(未使用)
WAIT_4X_13:
WAIT_4X_R11:
	nop								; 4 TIM0_CAPT(未使用)
WAIT_4X_12:
WAIT_4X_R10:
	nop								; 5 TIM0_OVF(未使用)
WAIT_4X_11:
WAIT_4X_R9:
	subi TMP, 1		; 4				; 6 TIM0_COMPA(未使用)
	sbci TMP2, 0	; 5				; 7 TIM0_COMPB(未使用)
	brcc WAIT_4X_11	; 6				; 8 ANA_COMP(未使用)
WAIT_RET:
	ret				; 7				; 9 WDT(未使用)
					; 11

WAIT_3X_12:
WAIT_3X_R10:
	rjmp WAIT_3X_10					; 10 LVM(未使用)

	reti							; 11 ADC

WAIT_3X_11:
WAIT_3X_R9:
	nop
WAIT_3X_10:
WAIT_3X_R8:
	subi TMP, 1		; 4
	brcc WAIT_3X_10	; 5
	ret				; 6
					; 10


	; ==============================================================================
	; システムクロック切り替え
	; ==============================================================================
SUB_LOWCLOCK:
	; 低音向け 1/4  rcall, ret 込みで 11クロック
	ldi TMP, CLKPSR_CLK_LOW
SUB_CHGCLOCK:
	out CCP, CONST_CCPID
	out CLKPSR, TMP
	ret


	; ==============================================================================
	; 初期化
	; ==============================================================================
RESET:
	; RESET PIN, BTNIN プルアップ
	ldi TMP, (1 << PNO_RESET) | (1 << PNO_BTNIN)
	out PUEB, TMP

	; 定数初期化
	ldi ZERO, 0
	ldi CONST_CCPID, 0xd8

	; SP設定
	ldi TMP, LOW(RAMEND)
	out SPL, TMP

	; SOUND を OUT/L、BTNOUT を OUT/L に
	out PORTB, ZERO ; 全出力ポート L
	ldi TMP, (1 << PNO_SOUND) | (1 << PNO_BTNOUT)
	out DDRB, TMP

	; ADC
	ldi TMP, PNO_BTNIN
	out ADMUX, TMP	; ADC 入力ピン設定

	; アナログコンパレータ OFF
	sbi ACSR, ACD

	; ==============================================================================
	; メインループ
	;	パワーダウンしてボタンが押されるまで待つ
	; ==============================================================================
MAIN:
	; ボタン待機クロック
	ldi TMP, CLKPSR_CLK
	rcall SUB_CHGCLOCK

MAIN_LP1:
	; ADC/TIMER 電源OFF
	ldi TMP, PRR_OFF
	out PRR, TMP

	; SLEEPモード = パワーダウン
	ldi TMP, SMCR_POWERDOWN
	out CCP, CONST_CCPID
	out SMCR, TMP

	sei

	; INT0 割込み許可
	sbi EIFR, INT0
	sbi EIMSK, INT0
	; パワーダウン
	sleep
	cbi EIMSK, INT0	; INT0割込み禁止

	; ボタン読み取り
	rcall SUB_READ_BUTTON
	cpi ARG, (BTN_NUM + BTN_EXTRA)
	brcc MAIN_LP1 ; 不明なボタン

	; ボタン処理
	rcall SOUND_PROC

	rjmp MAIN

; ボタン処理
.include "button.asm"

; サウンド処理
.include "sound.asm"
